unit AlternationManager;

{
  Keeps track of Alternations, provides useful UpdateAlternations and ApplyAlternations
  It is made as it will be inherited, never use it 'as is' (TAlternationManager.Create)

  Read UpdateAlternations and ApplyAlternations for more informations.

  Necessary for Group, Particle, ParticleSystem
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Alternation, DeletionList, Change, MyTime;

type
  DL_Alternation = specialize TDeletionList<TAlternation>;

  TAlternationManager = class
    public
      constructor Create;
      destructor Destroy; override;

      // <=> to call AddAlternation(TAlternation.Create(nil, nil, ch))
      procedure AddChange(ch1 : TChange); overload;
      procedure AddChange(ch1,ch2 : TChange); overload;
      procedure AddChange(ch1,ch2,ch3 : TChange); overload;
      procedure AddChange(ch1,ch2,ch3,ch4 : TChange); overload;
      procedure AddChange(ch1,ch2,ch3,ch4,ch5 : TChange); overload;

      //adds TAlternation to Pool
      procedure AddAlternation(a1 : TAlternation); overload;
      procedure AddAlternation(a1, a2 : TAlternation); overload;
      procedure AddAlternation(a1, a2, a3 : TAlternation); overload;
      procedure AddAlternation(a1, a2, a3, a4 : TAlternation); overload;
      procedure AddAlternation(a1, a2, a3, a4, a5 : TAlternation); overload;

    protected
  //!this class should be inherited and I don`t want to have same names if not neccessary !

  //calls update to all alternations,
  //also set time_diff  difference between this and last call, so Apply knows how many seconds passed
      procedure UpdateAlternations;

  //apply all Alternations to {sender}
      procedure ApplyAlternations(sender : TObject = nil);

    private
      dlAl : DL_Alternation;
      t : TMyTime;
      time_diff : Real; //in seconds
  end;

implementation

constructor TAlternationManager.Create;
begin
  //Writeln('TAlternationManager.Create');
  dlAl := DL_Alternation.Create;
  t := TMyTime.Create;
end;

destructor TAlternationManager.Destroy;
begin
  if dlAl <> nil then dlAl.Free;
  if t <> nil then t.Free;
  inherited;
  //Writeln('TAlternationManager.Destroy');
end;

procedure TAlternationManager.UpdateAlternations;
begin
  dlAl.Update; //Updates and deletes (not needed) alternations
  time_diff := t.updateTime;
end;

procedure TAlternationManager.ApplyAlternations(sender : TObject);
var
  iter : pointer;
begin
  if(time_diff <> 0.0) then begin

    //foreach Alternation call Execute
    iter := dlAl.first;
    while iter <> nil do begin
      dlAl.getElement(iter).Execute(sender, time_diff);
      dlAl.next(iter);
    end;

  end;
end;

procedure TAlternationManager.AddChange(ch1 : TChange);
begin
  AddAlternation(TAlternation.Create(nil, nil, ch1));
end;

procedure TAlternationManager.AddChange(ch1,ch2 : TChange);
begin
  AddChange(ch1);
  AddChange(ch2);
end;

procedure TAlternationManager.AddChange(ch1,ch2,ch3 : TChange);
begin
  AddChange(ch1);
  AddChange(ch2);
  AddChange(ch3);
end;
procedure TAlternationManager.AddChange(ch1,ch2,ch3,ch4 : TChange);
begin
  AddChange(ch1);
  AddChange(ch2);
  AddChange(ch3);
  AddChange(ch4);
end;
procedure TAlternationManager.AddChange(ch1,ch2,ch3,ch4,ch5 : TChange);
begin
  AddChange(ch1);
  AddChange(ch2);
  AddChange(ch3);
  AddChange(ch4);
  AddChange(ch5);
end;

procedure TAlternationManager.AddAlternation(a1 : TAlternation);
begin
  if (a1 <> nil) and (dlAl <> nil) then dlAl.Add(a1);
  if (dlAl <> nil) then //Writeln('TAlternationManager -> DeletionList not initialized');
end;
procedure TAlternationManager.AddAlternation(a1, a2 : TAlternation);
begin
  AddAlternation(a1);
  AddAlternation(a2);
end;
procedure TAlternationManager.AddAlternation(a1, a2, a3 : TAlternation);
begin
  AddAlternation(a1);
  AddAlternation(a2);
  AddAlternation(a3);
end;
procedure TAlternationManager.AddAlternation(a1, a2, a3, a4 : TAlternation);
begin
  AddAlternation(a1);
  AddAlternation(a2);
  AddAlternation(a3);
  AddAlternation(a4);
end;
procedure TAlternationManager.AddAlternation(a1, a2, a3, a4, a5 : TAlternation);
begin
  AddAlternation(a1);
  AddAlternation(a2);
  AddAlternation(a3);
  AddAlternation(a4);
  AddAlternation(a5);
end;

end.

