unit BoolFunction;

{
  classes to create bool statements, necessary for Alternation class

  you can find the common logical function as True, False, And, Or


  ...and you can create your own by inheriting TBoolFunction class
  (class allows you to construct functions as you wish :) )
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, DeletionList, MyTime;

type

  TBoolFunction = class
    public
    //for deletion list
      procedure Evaluate; virtual;        //what is done in each step (usually nothing)
      function IsDead : Boolean; virtual; //if I`m dead (you dont need me anymore)

      // -> This is the important function
      // you can also ask some conditions on sender (evoker of Alternation)
      function Value(sender : TObject) : Boolean; virtual;  //value of this boolean statement
  end;

  //archaic
  //TLE_BoolFn = specialize TListElement<TBoolFunction>;
  //LL_BoolFn = specialize TLinkedList<TLE_BoolFn, TBoolFunction>;

  DL_BoolFn = specialize TDeletionList<TBoolFunction>;

 // -------------------- TBoolTrue ------------------
  //funny class :)
  TBoolTrue = class(TBoolFunction)
  end;

 // -------------------- TBoolFalse ------------------
  TBoolFalse = class(TBoolFunction)
    public
      function Value(sender : TObject) : Boolean; override;
  end;

 // -------------------- TBoolOnce --------------------------
 TBoolOnce = class(TBoolFunction) //called just once
   public
   constructor Create; //sets variable to false
   procedure Evaluate; override;
   function IsDead : Boolean; override;    //returns already_called
   function Value(sender : TObject) : Boolean; override;

   protected
     already_called : Boolean;
     last_step : Boolean;
 end;

 // -------------------- TBoolEachMs ---------------------------
 TBoolEachMs = class(TBoolFunction)
    public
      constructor Create(each_ms : Integer);   //each Each_ms will this function once return true
      destructor Destroy; override;
      procedure Evaluate; override;
      function Value(sender : TObject) : Boolean; override;

    protected
      time : TMyTime;
      delay : Integer;
      this_step : Boolean;
 end;

 // -------------------- TBoolForMs ---------------------------
 TBoolForMs = class(TBoolFunction)
    public
     //for next N miliseconds will it be called
      constructor Create(next_ms : Integer);
      function Value(sender : TObject) : Boolean; override;
      function IsDead : Boolean; override;

    protected
      time_end : Integer;
 end;

 // -------------------- TBoolList ------------------
 {
   base for TBoolAnd and TBoolOr ... in future maybe TBoolEkv
   links together more TBoolFunctions
 }

  TBoolList = class(TBoolFunction)
    public
      constructor Create(a, b : TBoolFunction); overload;
      constructor Create(a, b, c : TBoolFunction); overload;
      constructor Create(a, b, c, d : TBoolFunction); overload;
      constructor Create(a, b, c, d, e : TBoolFunction); overload;
      destructor Destroy; override;

      procedure Evaluate; override;
      procedure Add(x : TBoolFunction);

    protected
    //DeletionList allows to erase no more needed functions...
    // ..so if some always false function Dies (in TBoolAnd) then others can make it true :)
      dlBF : DL_BoolFn;
  end;

 // -------------------- TBoolAnd ------------------
  TBoolAnd = class(TBoolList)
    public
      function Value(sender : TObject) : Boolean; override;
  end;

 // -------------------- TBoolOr ------------------
  TBoolOr = class(TBoolList)
    public
      function Value(sender : TObject) : Boolean; override;
  end;

implementation

//---------------------- TBoolFunction ---------------------
               {
destructor TBoolFunction.Destroy;
begin
  inherited;
end;
                }

function   TBoolFunction.Value(sender : TObject) : Boolean;
begin
  Result := true;
end;

function TBoolFunction.IsDead : Boolean;
begin
  Result := false;  //invincible
end;
procedure TBoolFunction.Evaluate;
begin

end;

//---------------------- TBoolFalse --------------------

function TBoolFalse.Value(sender : TObject) : Boolean;
begin
  Result := false;
end;


// -------------------- TBoolOnce --------------------------
constructor  TBoolOnce.Create;
begin
  already_called := false;
  last_step := false;
end;
function TBoolOnce.Value(sender : TObject) : Boolean;
begin
  Result := (not already_called) or (not last_step);
  already_called := true;
end;
procedure TBoolOnce.Evaluate;
begin
  if already_called then last_step := true;
end;

function TBoolOnce.IsDead : Boolean;
begin
  Result := last_step;
end;

 // -------------------- TBoolEachMs ---------------------------
constructor TBoolEachMs.Create(each_ms : Integer);
begin
  time := TMyTime.Create;
  delay := each_ms;
  this_step := false;
end;
destructor TBoolEachMs.Destroy;
begin
  if time <> nil then time.Free;
end;
procedure TBoolEachMs.Evaluate;
begin
  this_step := false;

  if (time.diffTimeMs >= delay) then begin
    time.addTimeMs(delay);
    this_step := true;
  end;
end;
function TBoolEachMs.Value(sender : TObject) : Boolean;
begin
  Result := this_step;
end;

// -------------------- TBoolForMs ---------------------------

constructor TBoolForMs.Create(next_ms : Integer);
begin
  time_end := _Time + next_ms;
end;

function TBoolForMs.Value(sender : TObject) : Boolean;
begin
  Result := (not self.IsDead);
end;
function TBoolForMs.IsDead : Boolean;
begin
  Result := _Time > time_end;
end;


//---------------------- TBoolList ---------------------

constructor TBoolList.Create(a, b : TBoolFunction);
begin
  dlBF := DL_BoolFn.Create;
  dlBF.Add(a);
  dlBF.Add(b);
end;
constructor TBoolList.Create(a, b, c : TBoolFunction);
begin
  dlBF := DL_BoolFn.Create;
  dlBF.Add(a);
  dlBF.Add(b);
  dlBF.Add(c);
end;
constructor TBoolList.Create(a, b, c, d : TBoolFunction);
begin
  dlBF := DL_BoolFn.Create;
  dlBF.Add(a);
  dlBF.Add(b);
  dlBF.Add(c);
  dlBF.Add(d);
end;
constructor TBoolList.Create(a, b, c, d, e : TBoolFunction);
begin
  dlBF := DL_BoolFn.Create;
  dlBF.Add(a);
  dlBF.Add(b);
  dlBF.Add(c);
  dlBF.Add(d);
  dlBF.Add(e);
end;

destructor TBoolList.Destroy;
begin
  if dlBF <> nil then dlBF.Free;
  inherited;
end;


procedure TBoolList.Evaluate;
begin
  dlBF.Update;
end;
{
function   TBoolList.Value : Boolean;
begin
  Result := true;
end;

function TBoolList.IsDead : Boolean;
begin
  Result := dlBF.empty;
end;
}

procedure  TBoolList.Add(x : TBoolFunction);
begin
  if dlBf = nil then begin
    //Writeln('TBoolList -> DeletionList not initialized.');
    exit;
  end;
  if x <> nil then dlBF.Add(x);
end;

//----------------------------- TBoolAnd --------------

function TBoolAnd.Value(sender : TObject) : Boolean;
var
  p : pointer;
begin
  p := dlBF.first;
  Result := true;

  while p <> nil do
  begin
    Result := (Result and dlBF.getElement(p).Value(sender));
    dlBF.next(p);
  end;
end;

//----------------------------- TBoolOr --------------
{!copy paste!}
function TBoolOr.Value(sender : TObject) : Boolean;
var
  p : pointer;
begin
  p := dlBF.first;
  Result := false;

  while p <> nil do
  begin
    Result := (Result or dlBF.getElement(p).Value(sender));
    dlBF.next(p);
  end;
end;

end.

