unit Bullet;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Particle, BulletInfo, Vektor, Graphics, Pattern, ParticleSystem,
  ParticleSelector, Alternation, BoolFunction, ParticleChange, Constants;

type
 //should be in hurting group (and in collision killing)
 //, when it is killed, bullet_info will be pushed into BIS
 //so it can be realized in ParticleSystemForm
  TBullet = class(TParticle)
    public
      constructor Create;      //just for calling inherited constructors
      constructor Create(position : TVektor; bullet_info : TBulletInfo);
      destructor Destroy;override;

      procedure Draw(img : TBitmap); override;

    protected
      info : TBulletInfo;
  end;

implementation
constructor TBullet.Create;
begin
  inherited;  //particle, alternation manager
end;

constructor TBullet.Create(position : TVektor; bullet_info : TBulletInfo);
begin
  Create;
  pos.copy(position);

  pattern.Free;
  pattern := TPatternCircle.Create(3);
  pattern.border := RgbToColor(100,100,100);
  pattern.border := 0;

  health := 100;
  weight := 0.1;

  info := bullet_info;

  flags := [];
end;

destructor TBullet.Destroy;
begin
 //the explosion will be created a little bit before
  info.x:=Round(pos.x - vel.x * cnt_bullet_destroy_exp_diff);
  info.y:=Round(pos.y - vel.y * cnt_bullet_destroy_exp_diff);
  //info.x:=Round(pos.x);
  //info.y:=Round(pos.y);

  if(info.power > 0) then begin
    PS.AddAlternation(TAlternation.Create(
      TBoolForMs.Create(cnt_bullet_destroy_ms),
      TSelectorStaticBoolFn.Create(TBoolTrue.Create, nil),
      TChangeGravity.Create(cnt_bullet_destroy_power * Real(info.power), 1.0, nil, false) //power of gravity depends on particle weight (but acceleration only on senders weight !)
    ));
  end;

  BIS.Push(info);
  inherited;
end;

procedure TBullet.Draw(img : TBitmap);
var
V1, V2 : TVektor;
i : Integer;
rot : Real;
begin
  rot := vel.rot;

  V1 := TVektor.Create(Cos(rot + 3.141592653589), Sin(rot + 3.141592653589));
  V2 := TVektor.Create;

  if pattern <> nil then pattern.Draw(img, Round(pos.x), Round(pos.y));

  for i := 5 downto 2 do begin
    if(health >= i * 18) then break;

    V2.copy(V1);
    V2.scale(5 * (7-i));

    img.Canvas.Pen.Color := RgbToColor(255,100,100);
    img.Canvas.Brush.Color := RgbToColor(255,100,100);
    img.Canvas.Ellipse(
      Round(pos.x + V2.x)-(i div 2),
      Round(pos.y + V2.y)-(i div 2),
      Round(pos.x + V2.x)+(i div 2),
      Round(pos.y + V2.y)+(i div 2)
    );
  end;
end;


end.

