unit DeletionList;

  {
   _TElement must have methods
   -Evaluate (something done in each step)
   -IsDead   (returns if I`m alive, if false then he will be deleted in Update from list)
   => See TDeletionElement (just template)

   TDeletionList manages a community of elements, which can Die and can Born
     automatically (if element.IsDead, then it will be erased from list, also deleted if Erase(true) is called)

   you just want to call .Update method (and Add sometimes)
  }

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LinkedList;

type
  generic TDeletionList<_TElement> = class
    public
      constructor Create;
      //calls also the destructors of elements
      destructor Destroy; override;

      //adds new element to list
      procedure Add(x : _TElement);
      function Empty : Boolean;

      //clears the list !!It dont call the destructors!!
      procedure Clear;

      //evaluates all elements, calls _TEl.Evaluate;
      procedure Evaluate;
      //if _TEl.IsDead then this element will be erased from list
      //                 if also free_them = true then .Free is called on them
      procedure Erase(free_them : Boolean = false);
      //calls Evaluate and Erase(true)
      procedure Update;

      //for iterating thru list (I can say that is even easier as in my LinkedList :) )
      //sample:
      // iter : pointer;
      // iter := DL.first
      // while iter <> nil do begin
      // ...
      // DL.next(iter)
      // end;
      function first : pointer;
      procedure next(var iterator : pointer);
      function getElement(p_in_ll : pointer) : _TElement;
    private
      //implemented with pointer list, it is easier to pretype all the time...
      L : LL_pointer;

  end;

implementation

constructor TDeletionList.Create;
begin
  ////Writeln('TDeletionList.Create');
  L := LL_pointer.Create;
end;

destructor TDeletionList.Destroy;
var
  p, q : pointer;
  el : _TElement;
begin
 //for each element
  q := L.b;
  while q <> nil do begin
    p := q;
    L.plus(q);
    el := getElement(p);
    if pointer(el) <> nil then el.Free;    //destroy all elements
  end;
  L.Free;
  inherited;
end;

procedure TDeletionList.Add(x : _TElement);
begin
  if pointer(x) <> nil then L.push_back(pointer(x));
end;
function TDeletionList.Empty : Boolean;
begin
  Result := L.empty;
end;

procedure TDeletionList.Clear;
begin
  L.clear;
end;
procedure TDeletionList.Evaluate;
var
  p : pointer;
  el : _TElement;
begin
  ////Writeln('TDeletionList.Evaluate');

  p := L.b;
  while p <> nil do begin
    el := getElement(p);

    if not el.IsDead then begin;
      el.Evaluate;
    end;

    L.plus(p);
  end;
end;
procedure TDeletionList.Erase(free_them : Boolean);
var
  p, q : pointer;
  el : _TElement;
begin
  //Writeln('TDeletionList.Erase:begin');

  q := L.b;
  while q <> nil do begin
    p := q;
    L.plus(q);
    el := getElement(p);
    if pointer(el) = nil then L.erase(p)
    else if el.IsDead then begin
      //Writeln('Element will be deleted');
      if free_them then el.Free;
      L.erase(p);
    end;
  end;
  //Writeln('TDeletionList.Erase:end');
end;

procedure TDeletionList.Update;
begin
  //Writeln('DeletionList(', L.size , ')');
  Evaluate;
  Erase(true);
end;

function TDeletionList.first : pointer;
begin
  Result := L.b;
end;

procedure TDeletionList.next(var iterator : pointer);
begin
  L.plus(iterator);
end;


function TDeletionList.getElement(p_in_ll : pointer) : _TElement;
begin
  //blee..
  Result := _TElement((TLE_pointer(p_in_ll)).value);
end;

end.

