unit Group;

{
  Groups particles, so you can add alternation to whole group
  and you mustn`t add the same alternation to all of them.

  It is base fot ParticleSystem and extends AlternationManager.
  Groups are never deleted in ParticleSystem.
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Particle, AlternationManager;

type
  TGroup = class(TAlternationManager)   //can be called more describively as ParticleManager
    public
      constructor Create; overload;
      constructor Create(parent_group : TGroup); overload;
      //don`t delete particles, just clears list
      destructor Destroy; override;

      //called each step, for all particles evaluates all alternations in this group
      //Dead particles are cleared, so it is necessary to not delete them, only on the end of step, see ParticleSystem.Update
      procedure Update;

      procedure Clear; virtual;

      //adds particle into group
      procedure AddParticle(p1 : TParticle); overload;
      procedure AddParticle(p1, p2 : TParticle); overload;
      procedure AddParticle(p1, p2, p3 : TParticle); overload;
      procedure AddParticle(p1, p2, p3, p4 : TParticle); overload;
      procedure AddParticle(p1, p2, p3, p4, p5 : TParticle); overload;

      //copy particles from group
      //!!TO DO!!
      //procedure AddParticle(g1 : TGroup); overload;

      //please use only with selectors as const
      function getParticleDelList : DL_Particle;

    protected
      dlPa : DL_Particle;
      p_group : TGroup;
  end;

implementation


constructor TGroup.Create;
begin
  inherited;
  p_group := nil;
  dlPa := DL_Particle.Create;
end;

constructor TGroup.Create(parent_group : TGroup);
begin
  //Writeln('TGroup.Create');
  Create;
  p_group := parent_group;
end;

destructor TGroup.Destroy;
begin
  dlPa.Clear; //I dont want to delete them, they are deleted in ParticleSystem
  dlPa.Free;
  inherited;  //delete Alternations
  //Writeln('TGroup.Destroy');
end;

//called each step
procedure TGroup.Update;
var
  iter : pointer;
begin
  UpdateAlternations;

  //to all particles apply all alternations
  iter := dlPa.first;
  while iter <> nil do begin
    ApplyAlternations(dlPa.getElement(iter)); //receiver is Particle in the list
    dlPa.next(iter);
  end;

  dlPa.Erase;  //erase (not delete) dead particles
end;

procedure TGroup.Clear;
begin
  dlPa.Clear;
  //dlAl.Clear;
end;

procedure TGroup.AddParticle(p1 : TParticle);
begin
  if p1 <> nil then dlPa.Add(p1);
  if (p1 <> nil) and (p_group <> nil) then p_group.AddParticle(p1);
  ////Writeln('TGroup.AddParticle');
end;
procedure TGroup.AddParticle(p1, p2 : TParticle);
begin
  AddParticle(p1);
  AddParticle(p2);
end;
procedure TGroup.AddParticle(p1, p2, p3 : TParticle);
begin
  AddParticle(p1);
  AddParticle(p2);
  AddParticle(p3);
end;
procedure TGroup.AddParticle(p1, p2, p3, p4 : TParticle);
begin
  AddParticle(p1);
  AddParticle(p2);
  AddParticle(p3);
  AddParticle(p4);
end;
procedure TGroup.AddParticle(p1, p2, p3, p4, p5 : TParticle);
begin
  AddParticle(p1);
  AddParticle(p2);
  AddParticle(p3);
  AddParticle(p4);
  AddParticle(p5);
end;

//please use only with selectors as const
function TGroup.getParticleDelList : DL_Particle;
begin
  Result := dlPa;
end;

//copy particles from group
{
procedure TGroup.AddParticle(g1 : TGroup);
begin

end;
}
end.

