﻿unit LinkedList;

{
  STL ekvivalent for LinkedList (it have only the basic functions, iterators are thru pointers)

  Declaring a list is also a little bit complicated, because you need a help structure:
    TLE_Integer = specialize TListElement<Integer>
    LL_Integer = specialize TLinkedList<TLE_Integer, Integer>

  Iterating thru:
    p : pointer;
    p := ll.b; //pascal have begin, end as reserved words :)
    while p <> nil do begin
    ...     //you can use ll.getElement(p)
    ll.plus(p);
    end;

  For function list see body
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

type
  { TElement }
  //helper method
  generic TListElement<_TPrvok> = class   // parameter šablóny
    public
      constructor Create;
      destructor Destroy; override;
      value : _TPrvok;
    private
      next, previous : pointer;
  end;


  { TList }
  generic TLinkedList<_TLE, _T> = class
    private
      //moves pointer by (when x negative, then backwards)
      function move(var p : pointer ; x : Integer) : Boolean;

      //functions for properties, set will push, if list is empty
      procedure setFront(x : _T);
      function getFront : _T;
      procedure setBack(x : _T);
      function getBack : _T;

      {----------------------variables---------------------}
      first: pointer;
      last: pointer;
      length : Integer;

    public
      constructor Create;
      destructor Destroy; override; //truncates varibles, it dont call destructors on value

      procedure push_front(el : _T); //insert before first element
      procedure push_back(el : _T);  //insert after last element
      procedure pop_front;           //erase first element
      procedure pop_back;            //erase last element
      procedure erase(p : pointer);  //erase this concrete element
      procedure clear;               //erase all elements

      //iterator functions
      function plus(var p : pointer) : Boolean; overload;
      function plus(var p : pointer; x : Integer) : Boolean; overload;
      function minus(var p : pointer) : Boolean; overload;
      function minus(var p : pointer; x : Integer) : Boolean; overload;

      function getElement(p : pointer) : _T;

      //returns pointers to first and last element
      function b : pointer;
      function e : pointer;

      function stringme : string;          //works only with integers

      //returns size of list
      function size : Integer;
      function empty : Boolean;

      //write will create new element if list is empty
      property front : _T read getFront write setFront;
      property back : _T read getBack write setBack;
  end;

  //some usefull templates
  TLE_int = specialize TListElement<Integer>;
  LL_int = specialize TLinkedList<TLE_int, Integer>;
  TLE_pointer = specialize TListElement<pointer>;
  LL_pointer = specialize TLinkedList<TLE_pointer, pointer>;

implementation

// ----------------------- TListElement ----------------
  constructor TListElement.Create;
  begin

  end;

  destructor TListElement.Destroy;
  begin
    inherited;
  end;

// ----------------------- TLinkedList ----------------
  constructor TLinkedList.Create;
  begin
    first := nil;
    last := nil;
    length := 0;
    inherited;
  end;

  destructor TLinkedList.Destroy;
  begin
    clear;  //clears elements
    inherited;
  end;

  procedure TLinkedList.push_front(el : _T); //insert before first element
  var
    novy : _TLE;
  begin
    novy := _TLE.Create;
    novy.value := el;
    novy.previous := nil;
    novy.next := first;
    if first <> nil then begin
      (_TLE(first)).previous := pointer(novy);
      end;
    first := pointer(novy);
    if last = nil then begin
      last := first;
    end;

    ////Writeln('First el = ', (_TLE(first)).value);
    ////Writeln('Last el = ', (_TLE(last)).value);

    Inc(length, 1);
  end;

  //I don`t like copy paste but .... (from time to time)
  procedure TLinkedList.push_back(el : _T);
  var
    novy : _TLE;
  begin
    novy := _TLE.Create;
    novy.value := el;
    novy.previous := last;
    novy.next := nil;
    if last <> nil then begin
      (_TLE(last)).next := pointer(novy);
      end;
    last := pointer(novy);
    if first = nil then begin
      first := last;
    end;

    ////Writeln('First el = ', (_TLE(first)).value);
    ////Writeln('Last el = ', (_TLE(last)).value);

    Inc(length, 1);
  end;

  procedure TLinkedList.pop_front;           //erase first element
  var
    buf : pointer;
  begin
    //Writeln('TLinkedList.pop_front');
    buf := first;

    if first <> nil then begin;
      plus(first);
      if first = nil then last := nil
      else (_TLE(first)).previous := nil;

      (_TLE(buf)).Free;
      Dec(length,1);
    end;
  end;

  procedure TLinkedList.pop_back;            //erase last element
  var
    buf : pointer;
  begin
    //Writeln('TLinkedList.pop_back');
    buf := last;

    if last <> nil then begin;
      minus(last);
      if last = nil then first := nil
      else (_TLE(last)).next := nil;

      (_TLE(buf)).Free;
      Dec(length, 1);
    end;
  end;
  procedure TLinkedList.erase(p : pointer);  //erase this concrete element
  var
    help : _TLE;
  begin
    //Writeln('TLinkedList.erase');
    if p = nil then exit;
    help := (_TLE(p));
    if help.previous = nil then pop_front
    else
      begin
        if help.next = nil then pop_back
        else begin
          (_TLE(help.previous)).next := help.next;
          (_TLE(help.next)).previous := help.previous;
        end;
      end;
  end;

  procedure TLinkedList.clear;
  var
    buffer : pointer;
  begin
    while first <> nil do begin
      buffer := (_TLE(first)).next;
      (_TLE(first)).Free;
      first := buffer;
    end;

    first := nil;
    last := nil;
    length := 0;
  end;

  function TLinkedList.plus(var p : pointer) : Boolean; overload;
  begin
    Result := false;
    if p = nil then exit;

    p := (_TLE(p)).next;
    Result := true;
  end;
  function TLinkedList.plus(var p : pointer; x : Integer) : Boolean; overload;
  begin
    Result := move(p, x);
  end;
  function TLinkedList.minus(var p : pointer) : Boolean; overload;
  begin
    Result := false;
    if p = nil then exit;

    p := (_TLE(p)).previous;
    Result := true;
  end;
  function TLinkedList.minus(var p : pointer; x : Integer) : Boolean; overload;
  begin
    Result := move(p, -x);
  end;

  function TLinkedList.getElement(p : pointer) : _T;
  begin
    Result := (_TLE(p)).value;
  end;

  function TLinkedList.stringme : string;
  var
    //buffer : pointer;
    s : string;
  begin
    //buffer := first;
    Str(length, s);
    ////Writeln('StringMe:');
    ////Writeln('First el = ', (_TLE(first)).value);
    ////Writeln('Last el = ', (_TLE(last)).value);
    Result := 'LinkedList(' + s + '): ';
    {
    while buffer <> nil do begin
      Str((_TLE(buffer)).value, s);
      Result := Result + s + ', ';
      buffer := (_TLE(buffer)).next;
    end;
    }
  end;

  function TLinkedList.b : pointer;
  begin
    Result := first;
  end;
  function TLinkedList.e : pointer;
  begin
    Result := last;
  end;

  function TLinkedList.size : Integer;
  begin
    Result := length;
  end;

  function TLinkedList.empty : Boolean;
  begin
    Result := (size = 0);
  end;

  function TLinkedList.move(var p : pointer ; x : Integer) : Boolean;
  var
    dir : Boolean;
  begin
    Result := true;
    dir := (x >= 0);
    if not dir then x := -x;

    while x > 0 do begin
      if dir then Result := plus(p)
      else Result := minus(p);
      Dec(x, 1);

      if not Result then break;
    end;
  end;

  procedure TLinkedList.setFront(x : _T);
  begin
    if empty then push_front(x)
    else (_TLE(first)).value := x;
  end;
  function TLinkedList.getFront : _T;
  begin
    if not empty then Result := (_TLE(first)).value;
  end;
  procedure TLinkedList.setBack(x : _T);
  begin
    if empty then push_back(x)
    else (_TLE(last)).value := x;
  end;
  function TLinkedList.getBack : _T;
  begin
    if not empty then Result := (_TLE(last)).value;
  end;

end.

