unit Options;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls;

procedure LoadGameData;
procedure SaveGameData;

type
  { TForm2 }

  TPlayerData = class(TObject)
    public
      name : string;
      strength : Integer;
      agility : Integer;
      color : Integer;
  end;

  TColorPicker = class(TObject)
    public
      constructor Create(owner : TComponent; x,y : Integer; img_x:Integer=-1; img_y:Integer=-1);
      destructor Destroy; override;
      function Value : Integer;

      procedure SetColor(col : Integer);
    protected
      procedure Refresh(Sender : TObject);

      scroll : array[0..2] of TTrackBar;
      l_name : array[0..2] of TLabel;
      l_value: array[0..2] of TLabel;

      preview: TImage;
  end;

  TForm2 = class(TForm)
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

    procedure LoadGameDataToElements;
    procedure SaveGameDataFromElements;
  public
    options_size : Integer;
    options_caption : array[0..10] of string;
    edit : array[0..4] of array[1..10] of TEdit;
    edit_int : array[0..4] of Boolean;

    cap : array[0..4] of array[1..10] of TLabel;
    player_label : array[0..4] of TLabel;
    color_picker : array[0..4] of TColorPicker;
  end; 

var
  Form2: TForm2;
  OPT_player: array[0..4] of TPlayerData;   //Created and Freeed id TForm1

implementation

constructor TColorPicker.Create(owner : TComponent; x,y : Integer; img_x, img_y : Integer);
var
  i : Integer;
  dy : Integer;
  names : array[0..2] of string;
begin

  dy := 25;
  names[0] := 'Red';
  names[1] := 'Green';
  names[2] := 'Blue';

  for i:= 0 to 2 do begin
    scroll[i]:= TTrackBar.Create(owner);
    scroll[i].Parent := TWinControl(owner);
    scroll[i].Left := x + 30;
    scroll[i].Top := y + dy * i;
    scroll[i].Min := 0;
    scroll[i].Max := 255;
    scroll[i].Width := 128;
    scroll[i].Height := 24;
    scroll[i].TickStyle := tsNone;
    scroll[i].OnChange:= @self.Refresh;
    scroll[i].Position := 42;

    l_name[i] := TLabel.Create(owner);
    with l_name[i] do begin
      Parent := TWinControl(owner);
      Left := x;
      Top := y + dy * i;
      Caption := names[i];
    end;

    l_value[i] := TLabel.Create(owner);
    with l_value[i] do begin
      Parent := TWinControl(owner);
      Left := x + 160;
      Top := y + dy * i;
      Caption := '42';
    end;
  end;

  preview := TImage.Create(owner);
  with preview do begin
    Parent := TWinControl(owner);

    if(img_x < 0) then Left := x + 130
    else Left := img_x;
    if(img_y < 0) then Top :=  y + dy * 3
    else Top := img_y;

    Width := 20;
    Height := 20;
  end;

  self.Refresh(TObject(self));
end;

destructor TColorPicker.Destroy;
var
  i : Integer;
begin
  for i := 0 to 2 do begin
    scroll[i].Free;
    l_name[i].Free;
    l_value[i].Free;
  end;
  preview.Free;
end;

function TColorPicker.Value : Integer;
begin
  Result := RgbToColor(
    scroll[0].Position,
    scroll[1].Position,
    scroll[2].Position
  );
end;

procedure TColorPicker.SetColor(col : Integer);
begin
  self.scroll[0].Position := col mod 256;
  self.scroll[1].Position := (col div 256) mod 256;
  self.scroll[2].Position := (col div 65536) mod 256;
  self.Refresh(TObject(self));
end;

procedure TColorPicker.Refresh(Sender : TObject);
var
  i : Integer;
begin
  for i := 0 to 2 do begin
    l_value[i].Caption := IntToStr(scroll[i].Position);
  end;

  preview.Canvas.Pen.Color := 0;
  preview.Canvas.Brush.Color := self.Value;

  preview.Canvas.FillRect(1,1,preview.Width-1, preview.Height-1);
end;

//-------------------- { TForm2 } -------------------------------//

procedure TForm2.FormCreate(Sender: TObject);
var
  i, j : Integer;
  dx, dy : Integer;
  d_py : Integer;
  pad : Integer;
  caption_pad : Integer;
  player_caption_pad : Integer;
begin
  options_size := 3;
  options_caption[0] := 'Name';
  options_caption[1] := 'Speed up';
  options_caption[2] := 'Handling';
  edit_int[0] := false;
  edit_int[1] := true;
  edit_int[2] := true;

  d_py := 25;
  dx := 200;
  dy := 200;
  pad := 10;
  caption_pad := 90;
  player_caption_pad := 15;

  for i := 0 to 3 do begin  //for players
    player_label[i] := TLabel.Create(self);

    with player_label[i] do begin
      Parent := self;
      Font.Height := 24;
      Font.Name := 'Arial';
      Left := pad  + dx * (i mod 2) + 10;
      Top := pad   + dy * (i div 2);
      Caption := 'Player ' + IntToStr(i + 1);
    end;
    color_picker[i] := TColorPicker.Create(self,
    pad  + dx * (i mod 2),
    d_py* (options_size + 1) + pad + player_caption_pad + dy * (i div 2),
    player_label[i].Left + 100,
    player_label[i].Top);

    for j := 0 to options_size - 1 do begin //for fields
      edit[i][j] := TEdit.Create(self);
      with edit[i][j] do begin
        Parent := self;
        Left := caption_pad + pad + dx * (i mod 2);
        Top :=  d_py* (j + 1) + pad + player_caption_pad + dy * (i div 2);
      end;
      cap[i][j] := TLabel.Create(self);
      with cap[i][j] do begin
        Parent := self;
        Left := pad + dx * (i mod 2);
        Top :=  d_py* (j + 1) + pad + player_caption_pad+ dy * (i div 2);
        Caption := options_caption[j];
      end;
    end;
  end;
end;

procedure TForm2.Button1Click(Sender: TObject);
begin
  SaveGameDataFromElements;
  SaveGameData;
  Close;
end;

procedure TForm2.FormDestroy(Sender: TObject);
var
  i, j : Integer;
begin
  for i := 0 to 3 do begin
    for j := 0 to options_size - 1 do begin
      edit[i][j].Free;
      cap[i][j].Free;
    end;
  end;
  for i := 0 to 3 do begin
    player_label[i].Free;
    color_picker[i].Free;
  end;
end;

procedure LoadGameData;
var
  subor : TextFile;
  i : Integer;
begin
  //!!TO DO!! - Load from file
  {
  for i := 0 to 3 do begin
    OPT_player[i].strength := 100;
    OPT_player[i].agility := 5;
  end;
  OPT_player[0].name := 'Petrzlen';
  OPT_player[1].name := 'Beren';
  OPT_player[2].name := 'Kombo';
  OPT_player[3].name := 'Edo';

  OPT_player[0].color := RgbToColor(0,0,0);
  OPT_player[1].color := RgbToColor(255,0,0);
  OPT_player[2].color := RgbToColor(0,255,0);
  OPT_player[3].color := RgbToColor(0,0,255);
  }

begin
  AssignFile(subor, 'options.txt');
  Reset(subor);

  for i := 0 to 3 do begin
    Readln(subor, OPT_player[i].name);
    Readln(subor, OPT_player[i].strength);
    Readln(subor, OPT_player[i].agility);
    Readln(subor, OPT_player[i].color);
  end;

  CloseFile(subor);
end;

end;

procedure SaveGameData;
var
  subor : TextFile;
  i : Integer;
begin
  AssignFile(subor, 'options.txt');
  Rewrite(subor);

  for i := 0 to 3 do begin
    Writeln(subor, OPT_player[i].name);
    Writeln(subor, OPT_player[i].strength);
    Writeln(subor, OPT_player[i].agility);
    Writeln(subor, OPT_player[i].color);
  end;

  CloseFile(subor);
end;

procedure TForm2.LoadGameDataToElements;
var
  i : Integer;
begin
  for i := 0 to 3 do begin
    self.edit[i][0].Text := OPT_player[i].name;
    self.edit[i][1].Text := IntToStr(OPT_player[i].strength);
    self.edit[i][2].Text := IntToStr(OPT_player[i].agility);
    self.color_picker[i].SetColor(OPT_player[i].color);
  end;
end;

procedure TForm2.SaveGameDataFromElements;
var
  i : Integer;
begin
  //!!TO DO!! - Save to file
  for i := 0 to 3 do begin
    OPT_player[i].name:= self.edit[i][0].Text;
    OPT_player[i].strength:= StrToInt(self.edit[i][1].Text);
    OPT_player[i].agility:= StrToInt(self.edit[i][2].Text);
    OPT_player[i].color:= self.color_picker[i].Value;
  end;
end;

initialization
  {$I options.lrs}

end.

