unit Particle;

{
  Saves data about particles. Also stores alternations which alter this particle.
  Main element of particle system.

  Extends TAlternationManager.
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Vektor, LinkedList, DeletionList, AlternationManager, Pattern, Graphics;

type
  //some states
  TParticleFlags = (static, invincible, highlight, has_blood);

  TParticle = class(TAlternationManager)
    public
      constructor Create; overload;
      constructor Create(x, y : Real; pat : TPattern); overload;
      destructor Destroy; override;

      //self-descriptive
      procedure KillMe;
      procedure HurtMe(x : Integer);

      //calls UpdateAlternations on self
      procedure Evaluate;                //for deletion list
      function  IsDead : Boolean;        //for deletion list

      procedure Draw(img : TBitmap); virtual;

   // ---------------- DATA ------------------------
      pos, vel : TVektor; //position and velocity
      health : Integer;

      pattern : TPattern; //my shape (ParticleSystem calls draw onPaint)

      col_index : Integer; //collision index
      weight : Real; //how hard is to stop me

      flags : set of TParticleFlags;
      parent : TParticle;
  end;

   //Templates
  TLE_Particle = specialize TListElement<TParticle>;
  LL_Particle =  specialize TLinkedList<TLE_Particle, TParticle>;
  DL_Particle =  specialize TDeletionList<TParticle>;

implementation

constructor TParticle.Create;
begin
  //Writeln('TParticle.Create');
  inherited;
  parent := nil;
  pos := TVektor.Create(0,0);
  vel := TVektor.Create(0,0);
  pattern := TPattern.Create;
  col_index := 0;

  health := 100;
  weight := 1.0;

  flags := [];
end;

constructor TParticle.Create(x, y : Real; pat : TPattern);
begin
  Create;    // -> no iherited needed, in TParticle.Create it is called
  pos.x := x;
  pos.y := y;
  pattern.Free;
  pattern := pat;
  pattern.border:=-1;
end;

destructor TParticle.Destroy;
begin
  if pos <> nil then pos.Free;
  if vel <> nil then vel.Free;
  if pattern <> nil then pattern.Free;
  inherited;  //destroys alternations

  //Writeln('Particle Destructor');
end;

function TParticle.IsDead : Boolean;
begin
  Result := (pos = nil) or (vel = nil) or (health <= 0);
end;
procedure TParticle.KillMe;
begin
  health := -1;
end;
procedure TParticle.HurtMe(x : Integer);
begin
  Dec(health, x);
end;

procedure TParticle.Evaluate;
begin
  if self.parent <> nil then begin
    if self.parent.IsDead then self.parent := nil;
  end;
  UpdateAlternations;
  ApplyAlternations(self);  //I`m the receiver

  //Writeln('Hello boys, I`m on position ', pos.str, ' and my velocity is ', vel.str);
end;
procedure TParticle.Draw(img : TBitmap);
begin
  if pattern <> nil then pattern.Draw(img, Round(pos.x), Round(pos.y));
end;

end.

