unit ParticleEmitor;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Group, Particle, Graphics, Vektor, Pattern, Alternation, Change,ParticleChange;

type
  TParticleEmitor = class(TObject)
  public
    constructor Create(emit_into: TGroup);
    //Par1, Par2 act as parameters (receiver, sender) in TChangeEmit
    function Emit(Par1: TParticle = nil; Par2: TParticle = nil): TParticle; virtual;
  protected
    g_into: TGroup;
  end;

  //From Change
  TChangeEmit= class(TChange)
    public
      constructor Create(particle_emitor : TParticleEmitor);
      procedure Apply(receiver:TObject; sender:TObject=nil; time_diff : Real = 0); override;

    protected
      emitor : TParticleEmitor;
  end;

  TParticleEmitorBlood = class(TParticleEmitor)
  public
    function Emit(Par1: TParticle = nil; Par2: TParticle = nil): TParticle;
      override;
  end;

implementation

//-----------------------   TParticleEmitor  --------------------------
constructor TParticleEmitor.Create(emit_into: TGroup);
begin
  g_into := emit_into;
end;

function TParticleEmitor.Emit(Par1: TParticle; Par2: TParticle): TParticle;
begin
  Result:=nil;
end;

//------------------------------- TChangeEmit ------------------------------
constructor TChangeEmit.Create(particle_emitor : TParticleEmitor);
begin
  emitor := particle_emitor;
end;
procedure TChangeEmit.Apply(receiver:TObject; sender:TObject=nil; time_diff : Real = 0);
begin
  emitor.Emit(TParticle(receiver), TParticle(sender));
end;

//-----------------------   TParticleEmitorBlood  --------------------------
function TParticleEmitorBlood.Emit(Par1: TParticle; Par2: TParticle): TParticle;
var
  diff: TVektor;
  i : Integer;
  b_num : Integer;
begin

  if (Par1 <> nil) and (Par2 <> nil) then
  begin
    if (has_blood in Par1.flags) then begin
      b_num := ((100 - Par1.health) div 10) + 2;
      for i := 0 to b_num do begin   //result act as buffer
        Result := TParticle.Create(Par2.pos.x, Par2.pos.y, TPatternCircle.Create(2));
        Result.pattern.inards := RgbToColor(255, 0, 0);
        Result.health := 3 + Random(10);
        Result.col_index := 0;
        Result.vel.move_polar_random(150.0, 1000.0);
        g_into.AddParticle(Result);
      end;

      Result := nil;
      if (Random(b_num) > 1) then begin
      diff := TVektor.Create(Par2.pos);
      diff.move(Par1.pos, -1.0);

      Result := TParticle.Create(Par1.pos.x, Par1.pos.y, TPatternCircle.Create(3));
      Result.pattern.inards := RgbToColor(255, 0, 0);
      Result.parent := Par1;
      Result.health := 400;
      Result.AddAlternation(TAlternation.Create(
        nil, nil, TChangeFollowParticle.Create(diff, 0.0))
      );

      g_into.AddParticle(Result);

      end;
    end;
  end;
end;

end.

