unit ParticlePairBoolFunction;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, PairBoolFunction, Particle, Pattern, Vektor;

//returns whether two particle collide
function CollideParticles(Par1, Par2 : TParticle) : Boolean;

type
  TBoolCollision = class(TPairBoolFunction)
    public
      function Value(receiver, sender : TObject) : Boolean; override;
  end;

implementation

// ------------------------- Collision functions ---------------
//returns some points where two patterns collide (if both coords negative, then no coliision)

// --------- Point, Point
function Collision(
  const S1 : TPatternPoint;
  const S2 : TPatternPoint;
  const P1, P2 : TVektor) : Boolean;overload;      //positions
begin
  if P1.same(P2) then Result := true
  else Result := false;
end;

// --------- Circle, Point
function Collision(
  const S1 : TPatternCircle;
  const S2 : TPatternPoint;
  const P1, P2 : TVektor) : Boolean;overload;      //positions
begin
  if P1.dist(P2) <= S1.r then Result := true
  else Result := false;
end;

// --------- Rect, Point
function Collision(
  const S1 : TPatternRectangle;
  const S2 : TPatternPoint;
  const P1, P2 : TVektor) : Boolean;overload;      //positions
begin
  if S1.IsIn(P2.x, P2.y, P1.x, P1.y)
    then Result := true
  else Result := false;
end;

// --------- Circle, Circle
function Collision(
  const S1, S2 : TPatternCircle;
  const P1, P2 : TVektor) : Boolean;overload;
begin
  if (P1.dist(P2) <= S1.r + S2.r) then begin
  Result := true;
  end
  else Result := false;
end;

// --------- Rect, Circle
function Collision(
  const S1 : TPatternRectangle;
  const S2 : TPatternCircle;
  const P1, P2 : TVektor) : Boolean;overload;      //positions
var
  help1, help2 : TPatternRectangle;
begin
  //!! TO DO !!
  help1 := TPatternRectangle.Create(S1.w + 2*S2.r, S1.h);
  help2 := TPatternRectangle.Create(S1.w, S1.h + 2*S2.r);

  if
   (help1.IsIn(P2.x, P2.y, P1.x, P1.y)
   or help2.IsIn(P2.x, P2.y, P1.x, P1.y)
   or (P2.dist(P1.x + S1.left, P1.y + S1.top) <= S2.r)
   or (P2.dist(P1.x + S1.right, P1.y + S1.bottom) <= S2.r)
   or (P2.dist(P1.x + S1.right, P1.y + S1.top) <= S2.r)
   or (P2.dist(P1.x + S1.left, P1.y + S1.bottom) <= S2.r)
   )
   then Result := true
  else Result := false;

  help1.Free;
  help2.Free;
end;


// --------- Rect, Rect
function Collision(
  const S1 : TPatternRectangle;
  const S2 : TPatternRectangle;
  const P1, P2 : TVektor) : Boolean;overload;      //positions
begin
  if ((S1.left + P1.x < S2.right + P2.x) and
     (S2.left + P2.x < S1.right + P1.x) and
     (S1.top + P1.y < S2.bottom + P2.y) and
     (S2.top + P2.y < S1.bottom + P1.y)
     ) then Result := true
  else Result := false;
end;

{ //template

function Collision(
  const S1 : TPattern;
  const S2 : TPattern;
  const P1, P2 : TVektor) : TVektor;overload;      //positions
begin
  //!! TO DO !!
  if false then Result := TVektor.Create(1,1)
  else Result := TVektor.Create(-42, -42);
end;

}

function CollideParticles(Par1, Par2 : TParticle) : Boolean;
var
  P1, P2 : TVektor;
  S1, S2 : TPattern;
  id1, id2 : Integer;
begin
  P1 := Par1.pos;
  P2 := Par2.pos;
  S1 := Par1.pattern;
  S2 := Par2.pattern;

  Result := true;
  if ((S1 = nil) or (S2 = nil)) then Result := false               //no shape
  else if (Par1.col_index <= 1) and (Par2.col_index <= 1) then Result := false; //no collision ability

  if Result then begin //no trivial cases
    id1 := S1.Id;
    id2 := S2.Id;

    if id1 = 1 then begin  //TPatternPoint
      if id2 = 1 then Result := Collision(TPatternPoint(S1),  TPatternPoint(S2) ,P1,P2);
      if id2 = 2 then Result := Collision(TPatternCircle(S2), TPatternPoint(S1) ,P2,P1);
      if id2 = 3 then Result := Collision(TPatternRectangle(S2),TPatternPoint(S1) ,P2,P1);
    end;
    if id1 = 2 then begin  //TPatternCircle
      if id2 = 1 then Result := Collision(TPatternCircle(S1), TPatternPoint(S2) ,P1,P2);
      if id2 = 2 then Result := Collision(TPatternCircle(S1), TPatternCircle(S2),P1,P2);
      if id2 = 3 then Result := Collision(TPatternRectangle(S2),   TPatternCircle(S1),P2,P1);
    end;
    if id1 = 3 then begin  //TPatternRect
      if id2 = 1 then Result := Collision(TPatternRectangle(S1), TPatternPoint(S2)   ,P1,P2);
      if id2 = 2 then Result := Collision(TPatternRectangle(S1), TPatternCircle(S2)  ,P1,P2);
      if id2 = 3 then Result := Collision(TPatternRectangle(S1), TPatternRectangle(S2)    ,P1,P2);
    end;
  end;
end;

//switch function for Collision functions group
// -1 -> TPattern
// 1  -> TPatternPoint
// 2  -> TPatternCircle
// 3  -> TPatternRectangle

function TBoolCollision.Value(receiver, sender : TObject) : Boolean;         //positions
begin
  Result := CollideParticles(TParticle(receiver), TParticle(sender));
end;

end.

