unit ParticleSelector;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Selector, ParticleSystem, Particle, BoolFunction, LinkedList, PairBoolFunction;

type
  {
    maybe a little bit abstract: you specify a bool function and
    this class on each step goes thru all particles and select all of them,
    which meet these conditions and they are not dead (and stores them)
  }
  TSelectorStaticBoolFn = class(TSelectorStatic)
    public
      //if the particle meets the bool function, then it will be returned
      constructor Create(bool_function : TBoolFunction; pair_function : TPairBoolFunction = nil);
      destructor Destroy; override;

      //on each particle in ParticleSystem tests the bfn condition
      procedure Evaluate;       override;           //called each step
      //returns bfn.IsDead
      function IsDead: Boolean; override;

    protected
      bfn : TBoolFunction;
  end; {TSelectorCollision}


implementation
//------------------------ TSelectorStaticBoolFn -------------------
constructor TSelectorStaticBoolFn.Create(bool_function : TBoolFunction; pair_function : TPairBoolFunction);
begin
  llPo := LL_pointer.Create;
  bfn := bool_function;
  pair_fn := pair_function;
end;
destructor TSelectorStaticBoolFn.Destroy;
begin
  if bfn <> nil then bfn.Free;
  inherited;
end;

//called each step
procedure TSelectorStaticBoolFn.Evaluate;
var
  dlPa : DL_Particle;
  par : TParticle;
  p : pointer;
begin
  bfn.Evaluate;
  llPo.clear;

  dlPa := PS.getParticleDelList;
  p := dlPa.first;

  while p <> nil do begin
    par := TParticle(dlPa.getElement(p));
    if not par.IsDead then begin
      //par := TParticle.Create;
      if bfn.Value(TObject(par)) then llPo.push_back(pointer(par));
    end;
    dlPa.next(p);
  end;
end;

function TSelectorStaticBoolFn.IsDead: Boolean;
begin
  Result := bfn.IsDead;
end;

end.

