unit ParticleSystem;

{
  Manages Groups, Particles ; it is just ancestor of TGroup, stores all particles.
  Also updates them, and deletes them when it is necessary.

  In ParticleChange is PS Declared, which is global.

  It extends TGroup and TAlternationManager.
  Groups are never deleted in ParticleSystem.

  //-------------How is Alternation Applied:  ----------------------
  //They are stored in ParticleSystem, Group, Particle
  //In Updates they are applied to all Particles in the group by ApplyAlternations(sender)
  //ApplyAlternations calls to all Alternation Alternation.Execute(sender, time_diff) , time_diff is computed in UpdateAlternations
  //Alternation.Execute calls BoolFn and Sender ---> calls ApplyTo(receiver, sender, time_diff)
  //ApplyTo foreach Change in Alternation calls Change.Apply(receiver, sender, time_diff)
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Group, Particle, LinkedList, MyTime, Graphics;

type
  TLE_Group = specialize TListElement<TGroup>;
  LL_Group  = specialize TLinkedList<TLE_Group, TGroup>;

  //class of classes, the 'BigBoss'
  //Just more methods
  TParticleSystem = class(TGroup)
    public
      constructor Create;
      destructor Destroy; override;

      //calls Updates for all Groups, Particles and Cleans up them
      procedure Update(clean_up : Boolean = true);
      procedure CleanUp;

      procedure Clear; override;

      //calls .shape.Draw to all particles
      procedure Draw(img : TBitmap);

      procedure AddGroup(g1 : TGroup);
    private
      lGr : LL_Group;    //can be implemented by vector, but I`m lazy to write one...
  end;

var
  PS : TParticleSystem;

implementation

constructor TParticleSystem.Create;
begin
  //Writeln('TParticleSystem.Create');
  lGr := LL_Group.Create;
  inherited;
end;

destructor TParticleSystem.Destroy;
var
  iter : pointer;
begin
  //Writeln('TParticleSystem.Destroy');
  dlPa.Free; //I want to delete Particles
  dlPa := DL_Particle.Create; //group destructor will call dlPa.Free

  //delete Groups
  iter := lGr.b;
  while iter <> nil do begin
    (TLE_Group(iter)).value.Free;
    lGr.plus(iter);
  end;
  lGr.Free;

  inherited; //deletes Alternations (in AlternationManager)
end;

procedure TParticleSystem.Update(clean_up : Boolean);
var
  iter : pointer;
begin
  //Writeln('--------------------------------');
  //Writeln('Main Cycle - begin (time = ',_Time, ')');

  dlPa.Evaluate; //just updates them (calls alternations
  UpdateAlternations; //updates global alternations

  //apply global alternations to all particles
  iter := dlPa.first;
  while iter <> nil do begin
    //if not dead then apply alternations on them
    if not dlPa.getElement(iter).IsDead then ApplyAlternations(dlPa.getElement(iter));
    dlPa.next(iter);
  end;

  //apply all group alternations
  iter := lGr.b;
  while iter <> nil do begin
    (TLE_Group(iter)).value.Update;
    lGr.plus(iter);
  end;

  if clean_up then dlPa.Erase(true); //delete them just after they are erased from Groups

  //Writeln('Main Cycle - end (time = ',_Time, ')');
  //Writeln('--------------------------------');
end;
procedure TParticleSystem.CleanUp;
begin
  dlPa.Erase(true);
end;

procedure TParticleSystem.Clear;
begin
  //!! TO DO !!
end;

procedure TParticleSystem.Draw(img : TBitmap);
var
  iter : pointer;
  par : TParticle;
begin
  iter := dlPa.first;
  while iter <> nil do begin

    par :=  dlPa.getElement(iter);
    if not par.IsDead then begin
      par.Draw(img);
    end;

    dlPa.next(iter);
  end;
end;

procedure TParticleSystem.AddGroup(g1 : TGroup);
begin
  if g1 <> nil then lGr.push_back(g1);
end;

end.

