unit ParticleSystemForm;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, Math,

  Particle, BoolFunction, MyTime, Group, Alternation, ParticleSystem,
  Pattern,
  ParticleChange, ParticleBoolFunction, ParticleSelector,
  Player, BulletInfo, ParticlePairBoolFunction, Weapon, Vektor, ParticleEmitor,

  MyGraphics, Options, Constants
  ;

var
  end_game : Boolean;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Image1: TImage;
    Image2: TImage;
    Memo1: TMemo;
    Timer1: TTimer;
    Timer2: TTimer;
    Timer3: TTimer;
    procedure EasyPopup(str : string);
    procedure WinnerPopup(winner_name : string);

    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Timer1Timer(Sender: TObject);   //for redrawing the screen
    procedure Timer2Timer(Sender: TObject);   //for updateing the _Time
    procedure Timer3Timer(Sender: TObject);   //for updateing the ParticleSystem

    procedure ReDraw;
  private
    { private declarations }
  end; 

var
  Form1: TForm1;
    g_exp : TGroup;
    g_gravity : TGroup;
    g_player : TGroup;
    g_obstacle : TGroup;
    g_killing : TGroup;
    player_num : Integer;
    player_dead : array[0..4] of Boolean;
    player_control : array[0..4] of array[0..10] of Char;
    players : array[0..4] of TPlayer;
    keystates : array[0..256] of Boolean;

    spawned_time : Integer;
    interrupt : Boolean;

implementation

{ TForm1 }

//negative will be considered as random
procedure SpawnObstacle(pos_x : Integer = -1 ; pos_y : Integer = -1 );
var
  par : TParticle;
  par_pat : TPattern;
begin
  if pos_x < 0 then pos_x := Random(Form1.Image1.Width);
  if pos_y < 0 then pos_y := Random(Form1.Image1.Width);

  if Random(2) = 0 then par_pat := TPatternCircle.Create(Random(46) + 4)
  else par_pat := TPatternRectangle.Create(Random(93) + 7, Random(93) + 7);

  par := TParticle.Create(
    pos_x,
    pos_y,
    par_pat
  );

  par.vel.x := Random(81) - 40;
  par.vel.y := Random(81) - 40;

  par.health := Round(par.pattern.Area / 50.0) + 10;
  par.pattern.border := -1;
  par.pattern.inards := Random(256*256*256);
  par.col_index:=2;
  par.weight := par.pattern.Area / 100.0;

  g_obstacle.AddParticle(par);
end;

//group should have some killing Alternation (so it will take health each step)
procedure Explosion(insert_into : TGroup; pos_x,pos_y : Integer;
                   par_number : Integer;
                   power : Integer);
var
  i : Integer;
  par : TParticle;
  r,g,b : Integer;
  health : Integer;
  w : Real;
begin
  for i := 1 to par_number do begin
    w := Real(Random(Round(Log2(power))) + 2);
    par := TParticle.Create(pos_x, pos_y, TPatternCircle.Create(Round(w)));
    insert_into.AddParticle(par);

    par.vel.move_polar_random(power div 4, (power * 3) div 4);
    health := Random(Round(Log2(power)) * 10);
    par.health:= health;
    par.col_index := 0;
    par.weight := w / 10.0;

    par.pattern.border := -1;
    if (power < 256) then r := Random(255 - power) + power
                     else r := 255;
    g := (r * 6) div 8;
    b := (r * 1) div 8;
    par.AddAlternation(TAlternation.Create(
      nil,
      nil,
      TChangeColorDependOnHealth.Create(r,g,b, r div 2,0,0, health, 0)
    ));

  end; {end of for : par_number}
end;

procedure BulletInfoUpdate;
var
  b_info : TBulletInfo;
  par : TParticle;
begin
  if not end_game then begin
    while not BIS.Empty do begin
      b_info := BIS.Pop;
      Form1.Memo1.Lines.Append('Explosion [' + IntToStr(b_info.x) + ', ' + IntToStr(b_info.y) + ']');

      case b_info.fn_id of
        1 : begin
          Explosion(g_exp, b_info.x, b_info.y, b_info.power, b_info.power * 20);
        end;
        2 : begin
          par := TParticle.Create(b_info.x, b_info.y,
            TPatternCircle.Create(10));
          par.weight := cnt_black_hole_weight;
          par.pattern.inards := 0;
          par.pattern.border := -1;
          par.health := cnt_black_hole_health;
          par.col_index := 0;
          par.flags := [static];
          g_gravity.AddParticle(par);
        end;
        3 : begin
          SpawnObstacle(b_info.x, b_info.y);
        end;
      end;
      b_info.Free;
    end;
  end;
end;

procedure InitWeapons;
var
  i : Integer;
  subor : TextFile;
begin

  WA_size := 4;
  for i := 0 to WA_size - 1 do begin WA[i] := TWeapon.Create; end;

//gun
  WA[0].name            := 'Gun';
  WA[0].explosion_id    := 1;       //Explosion, BlackHole
  WA[0].explosion_power := 5;      //power parameter of that function
  WA[0].velocity        := 300.0;   //velocity of bullet (pixels / second)
  WA[0].recoil          := 10.0;    //spetny raz, pixels / second
  WA[0].duration        := 0.7;     //in seconds
  WA[0].bullet_color    := RgbToColor(255,0,0); //color of launched bullet
  WA[0].reload          := 0.06;     //in seconds

//rifle
  WA[1].name            := 'Rifle';
  WA[1].explosion_id    := 1;
  WA[1].explosion_power := 30;
  WA[1].velocity        := 1000.0;
  WA[1].recoil          := 100.0;
  WA[1].duration        := 1.0;
  WA[1].bullet_color    := RgbToColor(0,255,0);
  WA[1].reload          := 1.0;

//gravity gun
  WA[2].name            := 'Gravity gun';
  WA[2].explosion_id    := 2;
  WA[2].explosion_power := 0;
  WA[2].velocity        := 40.0;
  WA[2].recoil          := 250.0;
  WA[2].duration        := 2.0;
  WA[2].bullet_color    := RgbToColor(0,0,0);
  WA[2].reload          := 2.0;

//obstacle gun
  WA[3].name            := 'Obstacle gun';
  WA[3].explosion_id    := 3;
  WA[3].explosion_power := 40;
  WA[3].velocity        := 1000.0;
  WA[3].recoil          := 500.0;
  WA[3].duration        := 0.2;
  WA[3].bullet_color    := RgbToColor(0,0,255);
  WA[3].reload          := 2.0;


  AssignFile(subor, 'weapons.txt');
  Reset(subor);

  Readln(subor, WA_size);

  //for i := 0 to WA_size - 1 do begin WA[i] := TWeapon.Create; end;

  for i := 0 to WA_size - 1 do begin
    Readln(subor, WA[i].name);
    Readln(subor, WA[i].explosion_id);
    Readln(subor, WA[i].explosion_power);
    Readln(subor, WA[i].velocity);
    Readln(subor, WA[i].recoil);
    Readln(subor, WA[i].duration);
    Readln(subor, WA[i].bullet_color);
    Readln(subor, WA[i].reload);
  end;

  CloseFile(subor);


                       {         //a template
  WA[3].name            := 'Obstacle gun';
  WA[].explosion_id    := ;
  WA[].explosion_power := ;
  WA[].velocity        := ;
  WA[].recoil          := ;
  WA[].duration        := ;
  WA[].bullet_color    := RgbToColor(,,);
  WA[].reload          := ;
  }
end;

procedure InitAlternations;
begin

//---------------------------- Particle System alternations -----------------------------
//movement
  PS.AddAlternation(TAlternation.Create(
    nil,
    nil,
    TChangeMove.Create
  ));
  //stay on screen

  PS.AddAlternation(TAlternation.Create(nil, nil,
    TChangeStayInBox.Create(0, 0, Form1.Image1.Width, Form1.Image1.Height)
    ,TChangeSpeedLimiter.Create(cnt_max_particle_speed, cnt_friction, 0.0)
  ));
  //collision

  PS.AddAlternation(TAlternation.Create(
    TBoolColIndex.Create(2),
    //TBoolFalse.Create,
    TSelectorStaticBoolFn.Create(TBoolColIndex.Create(1), TBoolCollision.Create),
    TChangeCollision.Create
  ));

//--------------------- killing field ---------------

  g_killing.AddAlternation(TAlternation.Create(
    TBoolEachMs.Create(10),
    nil,
    TChangeDamage.Create(1)
  ));

// --------------------- EXPLOSION -------------------------
  //damage explosion particles
  g_exp.AddAlternation(TAlternation.Create(
    TBoolEachMs.Create(10),
    nil,
    TChangeDamage.Create(1)
  ));

  g_exp.AddAlternation(TAlternation.Create(
    TBoolTrue.Create,
    //TBoolFalse.Create,
    TSelectorStaticBoolFn.Create(TBoolColIndex.Create(1), TBoolCollision.Create),
    //TChangeDamage.Create(10, true, false),
    //TChangeCollision.Create,   !! TO DO !! TChangePush (one side collision depend on weight)
    TChangeDamage.Create(1, true, false),
    TChangeEmit.Create(TParticleEmitorBlood.Create(g_killing)),
    TChangeKill.Create(false, true)
  ));

// ------------------------ OBSTACLES ----------------------------
  g_obstacle.AddAlternation(TAlternation.Create(
    nil,
    nil,
    TChangeColorDependOnHealth.Create(
      100, 200, 100,
      220, 100, 100,
      100, 0
    )
  ));


//------------------------- GRAVITATION --------------------------
  //killing field, drag here all particles
  g_gravity.AddAlternation(TAlternation.Create(
    TBoolEachMs.Create(1),
    nil,
    TChangeDamage.Create(1)
  ));

  g_gravity.AddAlternation(TAlternation.Create(
    TBoolTrue.Create,
    TSelectorStaticBoolFn.Create(TBoolTrue.Create, nil),
    TChangeGravity.Create(cnt_gravity_power, cnt_gravity_min_radius) //power of gravity depends on particle weight (but acceleration only on senders weight !)
  ));

end;

procedure InitPlayers;
var
  i,j, buf : Integer;
  cont_num : Integer;
  subor : TextFile;
begin
  //players
  for i:=0 to 4 do begin player_dead[i] := true; end;

  player_num := 4;
  for i:=0 to player_num-1 do player_dead[i] := false;

//-------------Controls-----------------------
  cont_num := 5;
  //Throttle, Left, Right, Fire, Change weapon
  AssignFile(subor, 'controls.txt');
  Reset(subor);

  for i := 0 to player_num - 1 do begin
    for j := 0 to cont_num - 1 do begin
      Readln(subor, buf);
      player_control[i][j] := Char(buf);
    end;
    Readln(subor);
  end;
  CloseFile(subor);

  for i := 0 to player_num - 1 do begin
    players[i] := TPlayer.Create;
    players[i].col_index:=2;
    players[i].name := OPT_player[i].name;
    players[i].pos.x := Random(Form1.Image1.Width);
    players[i].pos.y := Random(Form1.Image1.Height);

    players[i].pattern.Free;
    players[i].pattern := TPatternCircle.Create(10);
    players[i].pattern.inards:=OPT_player[i].color;
    players[i].agility:=Real(OPT_player[i].agility);
    players[i].strength:=Real(OPT_player[i].strength);

    players[i].flags := [has_blood];
    g_player.AddParticle(TParticle(players[i]));
  end;
end;

procedure CreateWorld;
var
  i : Integer;
begin
  //init ParticleSystem;
  PS := TParticleSystem.Create;
  g_killing := TGroup.Create(TGroup(PS));
  g_exp := TGroup.Create(TGroup(PS));
  g_player := TGroup.Create(TGroup(PS));
  g_obstacle := TGroup.Create(TGroup(PS));
  g_gravity := TGroup.Create(TGroup(PS));
  PS.AddGroup(g_killing);
  PS.AddGroup(g_exp);
  PS.AddGroup(g_player);
  PS.AddGroup(g_gravity);
  PS.AddGroup(g_obstacle);

  //keystates
  for i := 0 to 255 do begin keystates[i] := false; end;
//----------------PLAYERS---------------------
  InitPlayers;
//---------------------WEAPONS ---------------------
  InitWeapons;
//-----------------ALTERNATIONS-------------------
  InitAlternations;
//--------------- OBSTACLES------------------

  for i := 0 to 10 do SpawnObstacle;
  spawned_time := _Time;

end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i : Integer;
begin
  BIS := TBulletInfoStack.Create;
  randomize;
  _Time := 42;
  interrupt := false;

  PS := nil;

  for i:=0 to 3 do begin OPT_player[i] := TPlayerData.Create; end;
  LoadGameData;

  Image1.Canvas.Brush.Color := RgbToColor(255, 255, 255);
  Image1.Canvas.FillRect(Image1.ClientRect);

  Image2.Canvas.Brush.Color := RgbToColor(240,240,180);
  Image2.Canvas.FillRect(Image2.ClientRect);

  end_game := true;
end;

procedure TForm1.ReDraw;
var
  i, j, buf : Integer;
  play_rank : array[0..4] of Integer;
  play_rank_col : array[0..4] of Integer;

  dy : Integer;
begin
  if not end_game then begin
    for i:=0 to player_num - 1 do begin
      if not player_dead[i] then
        play_rank[i] := players[i].health
      else
        play_rank[i] := 0;
    end;
    for i:=0 to player_num - 2 do begin
      for j := i+1 to player_num -1 do begin
        if play_rank[i] < play_rank[j] then begin
          buf := play_rank[i];
          play_rank[i] := play_rank[j];
          play_rank[j] := buf;
        end;
      end;
    end;
    play_rank_col[0] := RgbToColor(240,240,180);
    play_rank_col[1] := RgbToColor(190,190,160);
    play_rank_col[2] := RgbToColor(140,140,140);
    play_rank_col[3] := RgbToColor(90,90,120);

    Image2.Picture.Bitmap.Canvas.Brush.Color := clWhite; //Random(256*256*256);
    Image2.Picture.Bitmap.Canvas.FillRect(Image1.ClientRect);
    dy := 90;

    for i:=0 to player_num - 1 do begin
      if player_dead[i] then begin
        Image2.Picture.Bitmap.Canvas.Pen.Color := 0;
        Image2.Picture.Bitmap.Canvas.Brush.Color := 0;
        Image2.Picture.Bitmap.Canvas.Rectangle(0, i*dy, Image2.Width, (i+1) * dy);
      end
      else begin
        for j :=0 to player_num-1 do begin
          if play_rank[j] = players[i].health then break;
        end;
        players[i].DrawInfo(Image2.Picture.Bitmap, i * dy, dy, play_rank_col[j]);
      end;
    end;

    DrawProgressBar(Image2.Picture.Bitmap, Real(spawned_time + cnt_spawn_time_diff - _Time), 10000.0,
      5, Image2.Height - 20, Image2.Width - 5, Image2.Height - 5
    );

    Image2.Canvas.Refresh;

    Image1.Picture.Bitmap.Canvas.Brush.Color := clWhite; //Random(256*256*256);
    Image1.Picture.Bitmap.Canvas.FillRect(Image1.ClientRect);
    PS.Draw(Image1.Picture.Bitmap);
    Image1.Canvas.Refresh;

  end;
end;

procedure DestroyWorld;
var
  i : Integer;
begin
  if not end_game then begin
    PS.Free;
    for i := 0 to WA_size - 1 do begin WA[i].Free; end;
    for i := 0 to player_num - 1 do begin player_dead[i] := true; end;
  end;

  SaveGameData;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  i : Integer;
begin
  DestroyWorld;

  while not BIS.Empty do begin
    BIS.Pop.Free;
  end;

  for i:=0 to 3 do begin OPT_player[i].Free end;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState
  );
var
  i : Integer;
begin
  keystates[Key] := true;

  Memo1.Lines.Append('Key id: ' + IntToStr(Integer(Key)));

  if not end_game then begin
    for i := 0 to player_num - 1 do begin
      if not player_dead[i] then begin
        //if(Key = Word('S')) then players[0].TurnBack;
        if(Key = Word(player_control[i][3])) then g_exp.AddParticle(players[i].LaunchBullet);
        if(Key = Word(player_control[i][4])) then players[i].ChangeWeapon;
      end;
    end;
  end;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  keystates[Key] := false;
end;

//draw scene
procedure TForm1.Timer1Timer(Sender: TObject);
begin
  if not end_game then begin
    ReDraw;
  end;
end;

//_Time update
procedure TForm1.Timer2Timer(Sender: TObject);
begin
  if not end_game then begin
    _Time := _Time + Integer(Timer2.Interval);
  end;
end;

//Particle system Update
procedure TForm1.Timer3Timer(Sender: TObject);
var
  i : Integer;
  alive : Integer;
  winer : Integer;
begin
//Winer check
  if not end_game then begin
    alive := 0;
    winer := 0;
    for i := 0 to player_num - 1 do begin
      if not player_dead[i] then begin
        Inc(alive, 1);
        winer := i;
      end;
    end;
    if alive <= 1 then begin
      if not interrupt then begin
        self.WinnerPopup(players[winer].name);
        DestroyWorld;
        end_game := true;
      end;
    end;
  end;

//Main loop
  if not end_game then begin
    BulletInfoUpdate; //creates explosions and so on
    PS.Update(false);
    for i:=0 to player_num - 1 do begin
      if not player_dead[i] then begin
        player_dead[i] := players[i].IsDead;
      end;
    end;
    PS.CleanUp;

    for i := 0 to player_num - 1 do begin
      if not player_dead[i] then begin
        if keystates[SmallInt(player_control[i][0])] then players[i].Accelerate(Timer3.Interval);
        if keystates[SmallInt(player_control[i][1])] then players[i].TurnLeft(Timer3.Interval);
        if keystates[SmallInt(player_control[i][2])] then players[i].TurnRight(Timer3.Interval);
      end;
    end;

    if(_Time >= spawned_time + cnt_spawn_time_diff) then begin
      SpawnObstacle();
      spawned_time := _Time;
      Memo1.Lines.Clear;
    end;
  //Memo1.Lines.Append(FormatFloat('###,###.##', players[0].rot));

  end;
end;


procedure TForm1.EasyPopup(str : string);
var
  F : TForm;
begin
  interrupt := true;

// Code copied from Andrej Blaho`s webpage; thank you Andrej!
    F := TForm.Create(self);
    with F do
    begin
      Name := 'EasyPopupHandler';
      Caption := 'Message';
      Width := 350;
      Height := 150;
      BorderStyle := bsSingle;
      Position := poScreenCenter;
      with TLabel.Create(f) do
      begin
        Name := 'Label';
        Parent := F;
        Font.Height := 16;
        Font.Name := 'Arial';
        Caption := str;
        Left := 20;
        Top := 16;
        AutoSize := True;
      end;
      with TButton.Create(f) do
      begin
        Name := 'Ok';
        Parent := F;
        Caption := 'Ok';
        Left := 132;
        Top := 120;
        Width := 75;
        Height := 25;
        ModalResult := mrOK;
      end;
      ShowModal;    // zobrazí nový formulár a čaká na jeho zatvorenie
      Free;         // zruší nový formulár
    end;

  interrupt := false;
end;

procedure TForm1.WinnerPopup(winner_name : string);
begin
  self.EasyPopup(
  'And the winner is: '#13 +
  winner_name + #13 +
  'Congratulations!'#13#13 +
  'More information about the program can be found at '#13 +
  'www.csip.sk');
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  DestroyWorld;
  end_game := true;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  end_game := false;
  CreateWorld;
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  buf_end_game : Boolean;
begin
  buf_end_game := end_game;
  end_game := true;

  Form2.LoadGameDataToElements;
  Form2.ShowModal;

  end_game := buf_end_game;
end;

initialization
  {$I particlesystemform.lrs}

end.

