unit Player;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Vektor, Particle, Graphics, Bullet, BulletInfo
  , Weapon, MyTime, MyGraphics;

type
   TPlayer = class(TParticle)
     public
       constructor Create;
       destructor  Destroy; override;

       procedure DrawPlayer(img : TBitmap; x,y : Real);
       procedure Draw(img : TBitmap); override;
       procedure TurnRight(power : Integer);  //power -> time_diff in miliseconds
       procedure TurnLeft(power : Integer);
       procedure TurnBack;

       procedure DrawInfo(img :TBitmap; dy, w : Integer; bg_color : Integer);

       procedure Accelerate(power : Integer);
       function LaunchBullet : TBullet;
       procedure ChangeWeapon;

       name : string;
       agility : Real;  //how fust can I turn
       strength   : Real;  //how fast can I accelerate
     protected
       weapon_id : Integer;
       rot : Real;

       t_reload: TMyTime;
       reload  : Real; //how much time I must wait to reload;
   end;

implementation
constructor TPlayer.Create;
begin
  inherited;
  reload := 0;
  t_reload := TMyTime.Create;
  weapon_id := 0;
  rot := 0.0;
  agility := 5.0;
  strength  := 500.0;
end;

destructor  TPlayer.Destroy;
begin
  inherited;
  t_reload.Free;
end;

procedure TPlayer.DrawPlayer(img : TBitmap; x,y : Real);
var
  V : TVektor;
begin
  reload := reload - t_reload.diffTime;
  t_reload.resetTime;

  V := TVektor.Create(Cos(rot), Sin(rot));
  V.scale(13);

  if pattern <> nil then pattern.Draw(img, Round(x), Round(y));

  img.Canvas.Brush.Color := RgbToColor(100,100,200);
  img.Canvas.Ellipse(
    Round(x + V.x)-2,
    Round(y + V.y)-2,
    Round(x + V.x)+2,
    Round(y + V.y)+2
  );

  V.Free;
end;

procedure TPlayer.Draw(img : TBitmap);
begin
  self.DrawPlayer(img, pos.x, pos.y);
end;
procedure TPlayer.TurnRight(power : Integer);  //power -> time_diff in miliseconds
begin
  rot := rot + ((Real(power) / 1000.0) * agility);
end;
procedure TPlayer.TurnLeft(power : Integer);
begin
  rot := rot - ((Real(power) / 1000.0) * agility);
end;
procedure TPlayer.TurnBack;
begin
  rot := rot + 3.14159265358979;
end;

procedure TPlayer.DrawInfo(img :TBitmap; dy, w : Integer; bg_color : Integer);
var
  padding : Integer;
begin
  padding := 5;

  img.Canvas.Pen.Color := 0;
  img.Canvas.Brush.Color := bg_color;
  img.Canvas.Rectangle(0, dy, img.Width, dy + w-1);

  self.DrawPlayer(img, 15.0, 15.0 + dy);
  img.Canvas.Pen.Color := 0;
  img.Canvas.Brush.Color := bg_color;
  img.Canvas.TextOut(padding + 30, padding + dy, self.name);

  //health bar
  DrawProgressBar(img, self.health, 100.0, padding, dy+30+padding, img.Width - padding, dy+40+padding);
  //reload bar
  DrawProgressBar(img, self.reload, 3.0, padding, dy + 40+2*padding, img.Width - padding, dy+50+2*padding, 0,0,255, 0,255,0);

  img.Canvas.Pen.Color := 0;
  img.Canvas.Brush.Color := bg_color;
  img.Canvas.TextOut(padding, dy + 50 + 3*padding, WA[weapon_id].name);
end;

function TPlayer.LaunchBullet : TBullet;
var
  V : TVektor;
begin

  if(reload <= 0.0) then
  begin
    V := TVektor.Create(pos);
    V.move_polar(rot, 15);

   Result := TBullet.Create(
      V,  //position
      TBulletInfo.Create(
        WA[weapon_id].explosion_id, WA[weapon_id].explosion_power
    ));
    Result.vel.copy(vel);
    Result.vel.move_polar(rot, WA[weapon_id].velocity);
    vel.move_polar(rot + 3.14159265, WA[weapon_id].recoil);

    Result.health := Round(WA[weapon_id].duration * 100.0);
    Result.pattern.inards := WA[weapon_id].bullet_color;

    reload := WA[weapon_id].reload;
  end
  else
    Result := nil;
end;
procedure TPlayer.ChangeWeapon;
begin
  Inc(weapon_id, 1);
  weapon_id := weapon_id mod WA_size;
end;

procedure TPlayer.Accelerate(power : Integer);
var
V : TVektor;
begin
  V := TVektor.Create(Cos(rot), Sin(rot));
  V.scale((Real(power) / 1000.0) * strength);
  vel.move(V);
  V.Free;
end;

end.

