unit Selector;

{
  usually selects some list of particles, so it must be implemented effectivelly:

  Select(sender)
  p : pointer;
  p := first;
  while p <> nil do begin
  ...
  next(p)
  end;
  Deselect;
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LinkedList, PairBoolFunction;

type
  TSelector = class
    public
      constructor Create(pair_function : TPairBoolFunction = nil);
      destructor Destroy; override;

      //for Deletion list (but I don`t know where it is used)
      procedure Evaluate; virtual;           //called each step
      function IsDead : Boolean; virtual;    //if true, then it kills whole Alternation

      //! this procedure will be overriden !
      //selects some objects, inserts to llPo, then user reads from it and clears me
      procedure Select(sender : TObject); virtual;

      //returns pointer to first TListElement<pointer> in selected list
      function first : pointer; virtual;
      //returns next element
      procedure next(var p : pointer); virtual;
      //also Apply needs TObject, returns value of that iterator
      function getElement(p : pointer) : TObject;

      //clears the list,
      //should be called on the end of selection, on the beginning of Select
      procedure DeSelect; virtual;

      pair_fn : TPairBoolFunction;
    protected
      llPo : LL_pointer;
  end; {TSelector}

  //changes only once by step ->
  //on Evaluate it selects particles
  //on Select sets iterator to beginning (and remembers sender)
  //on (first, next) skips Sender (must be remembered)
  TSelectorStatic = class(TSelector)
    public
      //in sucessors Evaluate must be overriden (and its enough to be done)

      //on Select sets iterator to beginning (and remembers sender)
      procedure Select(sender : TObject); override;

      //skips sender
      function first : pointer; override;
      //skips sender
      procedure next(var p : pointer); override;

      //just overrides Deselect (because default clears the list)
      procedure DeSelect; override;
    protected
      sen : TObject;
  end; {TSelectorStatic}

implementation

//---------------------------- TSelector ---------------------
constructor TSelector.Create(pair_function : TPairBoolFunction);
begin
  llPo := LL_pointer.Create;
  pair_fn := pair_function;
end;

destructor TSelector.Destroy;
begin
  if llPo <> nil then llPo.Free;
  if pair_fn <> nil then pair_fn.Free;
  inherited;
end;

procedure TSelector.Evaluate;
begin
  llPo.clear;
end;

function TSelector.IsDead : Boolean;
begin
  Result := false; //invincible man
end;

procedure TSelector.Select(sender : TObject);
begin

end;

function TSelector.first : pointer;
begin
  Result := llPo.b;
end;

procedure TSelector.next(var p : pointer);
begin
  llPo.plus(p);
end;

function TSelector.getElement(p : pointer) : TObject;
begin
  //blee..
  Result := TObject(llPo.getElement(p));
end;
procedure TSelector.DeSelect;
begin
  llPo.clear;
end;

//---------------------------- TSelectorStatic ---------------------


//on Select sets iterator to beginning (and remembers sender)
procedure TSelectorStatic.Select(sender : TObject);
begin
  sen := sender;
end;

//skips sender
function TSelectorStatic.first : pointer;
var
  p : pointer;
begin
  p := llPo.b;
  if (TObject(p) = sen) then p := nil;         //!! TO DO !! -> dont`t works, in Alternation Execute hard fixed
  Result := p;
end;
//skips sender
procedure TSelectorStatic.next(var p : pointer);
begin
  llPo.plus(p);
  if (TObject(p) = sen) then llPo.plus(p);   //!! TO DO !! -> dont`t works, in Alternation Execute hard fixed
end;

//just overrides TSelector.Deselect
procedure TSelectorStatic.DeSelect;
begin

end;

end.

