unit Vektor;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Math;

type
  TVektor = class
    public
      constructor Create(const V : TVektor); overload;
      constructor Create(a,b : Real); overload;
      function str : string;

      procedure move(mx, my : Real); overload;
      procedure move(const V : TVektor); overload;
      procedure move(const V : TVektor; scale : Real); overload;
      procedure move_polar(rot, r : Real);
      procedure move_polar_random(min, max : Real);

      procedure scale(r : Real);
      procedure scale_to(r : Real);

      function same(const V : TVektor) : Boolean;
      function dist(const V : TVektor) : Real; overload;
      function dist(a, b : Real) : Real;       overload;
      function size : Real;
      function rot : Real;

      procedure copy(const V : TVektor);

      x,y : Real;
    private
  end;

  //operator +(const V1: TVektor; const V2: TVektor) : TVektor;
  //operator *(const V1: TVektor; r : Real) : TVektor;
  //operator =(const V1, V2 : TVektor) : Boolean;

implementation
  constructor TVektor.Create(const V : TVektor);
  begin
    x := V.x;
    y := V.y;
  end;

  constructor TVektor.Create(a, b:Real);
  begin
    x := a;
    y := b;
  end;

  function TVektor.str : string;
  begin
    Result := '['+ FormatFloat('0.00', x)  +';'+ FormatFloat('0.00', y)  +']';
  end;
  function TVektor.same(const V : TVektor) : Boolean;
  begin
    Result := (x = V.x) and (y = V.y);
  end;

  function TVektor.dist(const V : TVektor) : Real;
  begin
    Result := Sqrt(Sqr(V.x-x) + Sqr(V.y-y));
  end;

  function TVektor.dist(a, b : Real) : Real;
  begin
    Result := Sqrt(Sqr(a-x) + Sqr(b-y));
  end;
function TVektor.size : Real;
begin
  Result := Sqrt(Sqr(x) + Sqr(y));
end;

  procedure TVektor.move(const V: TVektor);
  begin
    x := x + V.x;
    y := y + V.y;
  end;
  procedure TVektor.move(mx, my : Real);
  begin
    x := x + mx;
    y := y + my;
  end;
procedure TVektor.move(const V : TVektor; scale : Real);
begin
    x := x + (V.x * scale);
    y := y + (V.y * scale);
end;

  procedure TVektor.move_polar(rot, r : Real);
  begin
    self.move(Cos(rot) * r, Sin(rot) * r);
  end;
procedure TVektor.move_polar_random(min, max : Real);
var
  alpha, vel : Real;
begin
    alpha := Random(360000) div 1000;
    vel := min + Real(Random(Round((max-min) * 1000))) / 1000.0;
    self.move_polar(alpha, vel);
end;

  procedure TVektor.scale(r : Real);
  begin
    x := x * r;
    y := y * r;
  end;
procedure TVektor.scale_to(r : Real);
var
  s : Real;
begin
  s := self.size;
  if s <> 0.0 then self.scale(r / s);
end;

function TVektor.rot : Real;
begin
  Result := arctan2(y, x);
end;

procedure TVektor.copy(const V : TVektor);
begin
  x := V.x;
  y := V.y;
end;

end.

